/*
 * exempi - xmpconsts.h
 *
 * Copyright (C) 2007 Hubert Figuiere
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1 Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2 Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * 3 Neither the name of the Authors, nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software wit hout specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _EXEMPI_XMP_CONSTS_H_
#define _EXEMPI_XMP_CONSTS_H_

#ifdef __cplusplus
extern "C" {
#endif

extern const char NS_XMP_META[];
extern const char NS_RDF[];
extern const char NS_EXIF[];
extern const char NS_TIFF[];
extern const char NS_XAP[];
extern const char NS_XAP_RIGHTS[];
extern const char NS_DC[];
extern const char NS_EXIF_AUX[];
extern const char NS_CRS[];
extern const char NS_LIGHTROOM[];
extern const char NS_PHOTOSHOP[];
extern const char NS_CAMERA_RAW_SETTINGS[];
extern const char NS_CAMERA_RAW_SAVED_SETTINGS[];
extern const char NS_IPTC4XMP[];
extern const char NS_TPG[];
extern const char NS_DIMENSIONS_TYPE[];
/** Creative Commons namespace */
extern const char NS_CC[];
/* Added in Exempi 2.1 */
extern const char NS_PDF[];
/* Added in Exempi 2.5 */
extern const char NS_XML[];

#ifdef __cplusplus
}
#endif

#endif
